#ifndef FKT_H_
#define FKT_H_

#include <stdint.h>

template<typename T>
/** 
 * Converts a hex String into an integer. The given string is interpreted from position start to start+length-1 as a hex-value.
 * The given template should define a integer like type with Functions "*=", "+=" and "*". 
 * @param _str defines a c-string
 * @param _start defines the first char which has to be read. Standardvalue 0.
 * @param _length defines how many bits have to be read. Standardvalue -1 determines that the string is read to the end.
 */
T h2i(char* _str,int _start = 0 , int _length=-1)
{
	T zw=0;
	bool neg=false;
	uint32_t i=_start;
	if(_str[i]=='-')
	{
		i++;
		neg=true;
		_length--;
	}
	for(i; i&&_str[i]!=0 && _length!=0 ;i++)
	{
		zw*=16;
		zw += (_str[i]>='A')?(_str[i]-'A'+10):(_str[i]-'0');
		_length--;
	}
	return (neg?zw*(-1):zw);
}
template<typename T>
/** 
 * Converts a decimal String into an integer. The given string is interpreted from position start to start+length-1 as a dec-value.
 * The given template should define a integer like type with Functions "*=", "+=" and "*". 
 * @param _str defines a c-string
 * @param _start defines the first char which has to be read. Standardvalue 0.
 * @param _length defines how many bits have to be read. Standardvalue -1 determines that the string is read to the end.
 */
T d2i(char* _str,int _start = 0 , int _length=-1)
{
	T zw=0;
	bool neg=false;
	uint32_t i=_start;
	if(_str[i]=='-')
	{
		i++;
		neg=true;
		_length--;
	}
	for(i; i&&_str[i]!=0 && _length!=0 ;i++)
	{
		zw*=10;
		zw += _str[i]-'0';
		_length--;
	}
	return (neg?zw*(-1):zw);
}

/**
 * Sorts a list of pointers to elements in place.
 * @param _list The list of pointers to the elements. This list will be altered and contain the sorted list.
 * @param _length The length of the list.
 */
template<typename T>
void sort(T* _list, uint32_t _length)
{
	qs(_list, 0,_length-1);
}
/**
 * The quicksort for sorting. Sorts a part of the list.
 * @param _list the list that is going to be sorted.
 * @param _left The left border of the sublist.
 * @param _right The right border of the sublist. 
 */
template<typename T>
void qs(T* _list, int32_t _left, int32_t _right)
{
	if(_right-_left<1)
		return;
	T p=_list[_right];
	T sw;
	uint32_t i=_left-1;
	uint32_t j = _right;	
	do
	{
		do{
			i++;
		}while(_list[i]<p);
		do{
			j--;
		}while(j>_left&&_list[j]>p);
		if(j>i)
		{
			sw=_list[i];
			_list[i]=_list[j];
			_list[j]=sw;
		}
	}while(j>i);
	sw= _list[i];
	_list[i]=_list[_right];
	_list[_right]=sw;
	qs(_list,_left,i-1);
	qs(_list,i+1,_right);
}

/**
 * Processes a inputfile with respect to the average execution time.
 * Requires a experiment result file as input. Produces a latex processable file.
 * @param _ifiles A list of name of inputfiles that are going to be processed.
 * @param _ofile The name of the ouptutfile without fileextension.
 * @param _length The length of the inputfile list.
 * @param _fts The list of functiontypes of interrset.
 * @param _ft The length of the functiontype list.
 */
void medtimetex(char* _ifiles[], char* _ofile, uint32_t _length, uint32_t* _fts, uint32_t _ft);
/**
 * Processes a inputfile with repsect to the average number of swaps.
 * Requires a experiment result file as input. Produces gnuplot processable file.
 * @param _ifile The name of the inputfile.
 * @param _ofile The name of the outputfile, without fileextension.
 * @param _index The index number used in the gnuplot file. This variable will be updated.
 * @param _fts A list of funtiontypes of interrest.
 * @param _ft The length of the list of funtiontypes.
 */
void boxplotchanges(char* _ifile, char* _ofile, int &_index, uint32_t* _fts, uint32_t _ft);
/**
 * Processes a inputfile with repsect to the execution time.
 * Requires a experiment result file as input. Produces gnuplot processable file.
 * @param _ifile The name of the inputfile.
 * @param _ofile The name of the outputfile, without fileextension.
 * @param _index The index number used in the gnuplot file. This variable will be updated.
 * @param _fts A list of funtiontypes of interrest.
 * @param _ft The length of the list of funtiontypes.
 */
void boxplottime(char* _ifile, char* _ofile, int &_index, uint32_t* _fts, uint32_t _ft);

/**
 * Processes a inputfile with respect to the used time. 
 * Requires an experiment result file as input. Produces a gnuplot processable fileformat.
 * @param _ifile 	Defines the inputfile that is going to be processed.
 * @param _ofile 	Defines the name of the Outputfiles without fileextension.
 * @param _index	Indexnumber used in the gnuplotfile.
 * @param _fts		A list of functiontypes of interrest.
 * @param _ft		Length of the functiontype list.
 */
void time(char* _ifile, char* _ofile,int &_index, uint32_t* _fts, uint32_t _ft);

/**
 * Processes a inputfile with respect to the used stashsizes.
 * Requires an experiment result file as input.  Produces a gnuplot processable fileformat.
 * @param _ifile 	Defines the inputfile that is going to be processed.
 * @param _ofile 	Defines the name of the Outputfiles without fileextension.
 * @param _maxh		Defines the maximum Stashsize of interrest. .
 * @param _fts		A list of functiontypes of interest.
 * @param _ft		Length of thefunctiontype list.
 */
void stash(char* _ifile, char* _ofile, int _maxh, uint32_t* _fts, uint32_t ft);

/**
 * Creates the headline if the inputfiles are experimentresults.
 * @param _ifile 	The name of the inputfile
 * @param _ofile	The name of the outputfiles without fileextension
 */
void createHeadExperiments(char* _ifile, char* _ofile);

/**
 * Creates a table for the results of a function test.
 * Requires an function test result file as input. Produces a latex processable file
 * @param _ifiles	a list of the names of various Inputfiles
 * @param _ofile        the name of the outputfile without fileextension
 * @param _listlength	the number of file names in the _ifiles list
 * @param _fts		a list of functiontypes of interest
 * @param _ft		length of the functiontype list
 */
void functiontesttable(char* _ifiles[], char* _ofile, int _listlength, uint32_t* _fts, uint32_t _ft);

/**
 * Displays a help text.
 */
void help();
#endif
