#ifndef UNIVERSAL_H_
#define UNIVERSAL_H_

#include "HashFunction.h"


/** A class for 2 Universal hashfunctions.
 *  This class realizes 2 hashfunctions h1, h2 as 2 universal hashfunctions. 
 *  It is a shift based implementation where one Hashfunction has got the form (a*x)>>(l-l_out).
 */
class TwoUniversal:public HashFunction
{
public:
	/**
	* The two hashfunctions.
	* @param _key the number that is used for the calculation of the hashvalue. 
	*/
	sizetype h1 (keytype _key);
	sizetype h2 (keytype _key);

	/**
	* Constructor called by create function.
	* @param _hashsize defines the size of the hashtable for which the hashvalues are calculated
	*/
	TwoUniversal(sizetype _hashsize);

	/**
	 * Shows the maximum operatable key value.
	 * @returns the maximum operatable key value.
	 */
	universetype getmaxUniverse(){return 0xFFFFFFFF;};

	/**
	 * Defines the name of the Hashfunction.
	 * @returns a char pointer which contains the name of the Hashfunction.
	 */
	char* getSpecifier(){return const_cast<char*>("Shiftbased 2 Unviersal");};
private:

	/**
	* contains the number of bits that the result is going to be shifted. 
	* (l-l_out)
	*/
	sizetype        p_shift;
	/**
	 * The two parameters a for the two hashfunctions h1, h2.
	 */
	universetype	p_a1,p_a2;
};

#endif
