#ifndef CW_5_UNIVERSAL_H_
#define CW_5_UNIVERSAL_H_

#include <HashFunction.h>
#include <iostream>

/**
 * This class provides a 5-universal hashfunction using the cw trick.
 * This hashfunction is based on the description in:
 * "Tabulation-Based 5-Independent Hashing with Applications to Linear Probing
 *  and Second Moment Estimation",
 * M.Thorup,Y.Zhang, SIAM2012 
 */
class CW5Universal: public HashFunction
{
public:
	/**
	 * both hash functions.
	 */
      	sizetype h1(keytype _key);
      	sizetype h2(keytype _key);
	/**
	 * Constructor.
	 * @param _hashsize
	 */
      	CW5Universal(sizetype _hashsize);
	
	/**
	 * Provides information about the maximal usable universe.
	 */
      	universetype getmaxUniverse(){return p_prime-1;};
	
	/**
	 * gives acces to the used parameters in both functions.
	 * @param _cnt	will contain the length of the parameter list.
	 * @returns The list of parameters.
	 */
     	universetype* getParam(uint32_t& _cnt){
      	universetype * res = new universetype[12];
	_cnt=12;
	res[0]=p_a1;
	res[1]=p_b1;
	res[2]=p_c1;
	res[3]=p_d1;
	res[4]=p_e1;
	res[5]=p_a2;
	res[6]=p_b2;
	res[7]=p_c2;
	res[8]=p_d2;
	res[9]=p_e2;
	res[10]=p_prime;
	res[11]=p_m;
	return res;
      }
 	/**
	 * Returns a specifier string for this hash function.
	 * @returns a c-type string containing the specifier string.
	 */
      	char* getSpecifier(){return const_cast<char*>("CW-Trick 5 Universal");};
private:
	/**
	 * The parameters fpr both hashfunctions.
	 */
      universetype p_a1,p_b1, p_c1, p_d1, p_e1;
      universetype p_a2,p_b2, p_c2, p_d2, p_e2;

	/**
	 * Parameters common in both hash functions.
	 * The used prime and the maximal output value.
	 */
      universetype p_prime, p_m;
      
	/**
	 * Calculates a*x+b mod p. Using the CW trick.
	 * posibly +2*p or +p
	 */
      inline universetype CWTrick(keytype x, universetype a, universetype b);

      
};

#endif
