#ifndef DA_C_1_H_
#define DA_C_1_H_

#include <HashFunction.h>
#include <iostream>
/**
 * This class provides a hashfunction as proposed in 
 * "Explicit and Efficient Hash Families Suffice for Cuckoo Hashing with a Stash", 
 * M.Aumüller, M.Dietzfelbinger, P.Woelfel, ESA2012. 
 * With parameter c chosen to 1.
 */
class DAC1: public HashFunction
{
public:
	/**
	 * both hash functions
	 */
      sizetype h1(keytype _key);
      sizetype h2(keytype _key);
	/**
	 * Constructor.
	 * @param _hashsize 
  	 * @param _universesize
	 * @param _stashsize
	 * @param _delta
	 */
      DAC1(sizetype _hashsize, keytype _universesize, sizetype _stashsize=3, double _delta=0.5);
       /**
	* Destructor.
	*/	
      ~DAC1();
	/**
	 * provides information aboout the maximal usable universe
	 */
      universetype getmaxUniverse(){return 0xFFFFFFFF;};

     	/**
	 * Returns a specifier string for this hash funcion.
	 * @returns a c-type string containing the specifier string.
	 */ 
      char* getSpecifier(){return const_cast<char*>("Dietzfelbinger Aumüller c=1");};
private:
	/**
	 * The list of parameters for both hash functions.
	 */
      universetype* p_param1;
      universetype* p_param2;

	/**
	 * The maximal output value.
	 */
      universetype p_m;

	/**
	 * The length of the parameter lists.
	 */ 
      uint32_t p_j;

	/**
	 * The f function.
	 */
      HashFunction* p_f;
	/**
	 * The g function for access in the parameter lists.
	 */
      HashFunction* p_g;
 
     uint32_t round_up(double x);
};

#endif
