#ifndef DA_K_1_H_
#define DA_K_1_H_

#include <HashFunction.h>
/**
 * This class provides a hashfunction as proposed in 
 * "Explicit and Efficient Hash Families Suffice for Cuckoo Hashing with a Stash", 
 * M.Aumüller, M.Dietzfelbinger, P.Woelfel, ESA2012. 
 * With parameter k chosen to 1.
 */
class DAK1: public HashFunction
{
public:
	/**
	 * The first hashfunction.
	 * @param _key The key this function is evaluated for.
	 * @returns the to the key coressponding hashvalue.
	 */
      	sizetype h1(keytype _key);
	/**
	 * The second hashfunction.
	 * @param _key The key this funciton is evaluated for.
	 * @returns the to the key coressponding hashvalue.
	 */
      	sizetype h2(keytype _key);
	
	/**
	 * Constructor 
	 */
      	DAK1(sizetype _hashsize, keytype _universesize, sizetype _stashsize = 3, double _delta = 0.5);
      	/**
	 * Destructor
	 */
	~DAK1();

	/*
	 * Returns the maximal usable universe with this function.
	 * @returns the maximal usable universe with this function.
	 */
      	universetype getmaxUniverse(){return 0xFFFFFFFF;};

	/** 
	 * Returns a specifing string for this hashfunction.
	 * @returns a c-type string specifing this hashfunction.
	 */
      	char* getSpecifier(){return const_cast<char*>("Dietzfelbinger Aumüller k=1");};
      
private:
	/**
	 * A list of parameter lists for the first function.
	 */
      	universetype** p_param1;
	/**
	 * A list of parameter lists for the second function.
  	 */
      	universetype** p_param2;

	/**
	 * defines the range of both hashfucntions.
	 */
      	universetype p_m;

	/**
	 * Defines the number of parameter lists and correponding g-functions for each hashfunction.
	 */
      	uint32_t p_C;
	
	uint32_t p_j;
	
	
      	HashFunction* p_f;
      	HashFunction** p_g;
	
	/**
	 * Rounds the given number x up.
	 * @param x the number, that is going to be rounded up.
	 * @returns the next greater or equal integer value above x.
	 */
     	uint32_t round_up(double x);
};

#endif
