#ifndef DA_K_2_H_
#define DA_K_2_H_

#include <HashFunction.h>
#include <iostream>
/**
 * This class provides a hashfunction as proposed in 
 * "Explicit and Efficient Hash Families Suffice for Cuckoo Hashing with a Stash", 
 * M.Aumüller, M.Dietzfelbinger, P.Woelfel, ESA2012. 
 * With parameter k chosen to 2.
 */

class DAK2: public HashFunction
{
public:
	/**
	 * both hash functions
	 */
      sizetype h1(keytype _key);
      sizetype h2(keytype _key);
	/**
	 * Constructor.
	 * @param _hashsize
	 * @param _universeize
	 * @param _stashsize
	 * @param _delta
	 */
      DAK2(sizetype _hashsize, keytype _universesize, sizetype _stashsize=3, double _delta=0.5);
      	/**
	 * Destructor.
	 */
      ~DAK2();

	/**
	 * Provides information about the maximal usable key.
	 * @returns the maximal usable key.
	 */
      universetype getmaxUniverse(){return 0xFFFFFFFF;};
	
      	/**	
	 * Returns a specifier string for this hash function.
	 * @returns a c-type string containing this specifier string.
	 */
      char* getSpecifier(){return const_cast<char*>("Dietzfelbinger Aumüller k=2");};
private:
	/**
	 * The parameter lists used by the first hashfunction.
	 */
      universetype** p_param1;
	/**
	 * The parameter lists used by the second hashfunction.
	 */
      universetype** p_param2;

	/**	
	 * The maximal outputvalue.
	 */
      universetype p_m;

	/**
	 * the number of parameter lists for each hashfunction
	 */
      uint32_t p_C;
	/**
	 * the length of each parameter list.
	 */
      uint32_t  p_j;

	/**
	 * Hashfunctions for the direct calculated part of the hashfunction.
	 */
      HashFunction* p_f;
	/**
	 * Hashfunctions for acces to the parameter lists.
	 */
      HashFunction** p_g;
      

     uint32_t round_up(double x);
};

#endif
