#ifndef FULLY_RANDOM_H_
#define FULLY_RANDOM_H_
#include "HashFunction.h"
#include <set>

/**
 * Struct for storing the keys together with their hashvalues.
 */
struct KeyFunction
{
    
    /**
     * The stored key value
     */
    keytype key;
    /**
     * The stored hashvalue for hashfunction 1.
     */
    sizetype h1;
    /**
     * The stored hashvalue for hashfunciton 2.
     */
    sizetype h2;
    /**
     * Constructor.
     */
    KeyFunction(keytype _key):key(_key){}
    KeyFunction(keytype _key, sizetype _h1, sizetype _h2):key(_key),h1(_h1),h2(_h2){}
    /**
     * Comparsion operators. Only dependend on the key value.
     */
    inline bool operator<(const KeyFunction& _ref) const
    {
        return (key<_ref.key);
    }
    inline bool operator<=(const KeyFunction& _ref) const
    {
        return (key<=_ref.key);
    }
    inline bool operator >(const KeyFunction& _ref) const
    {
        return (key>_ref.key);
    }
    inline bool operator>=(const KeyFunction& _ref) const
    {
        return (key>=_ref.key);
    }
    inline bool operator==(const KeyFunction& _ref) const
    {
        return (key==_ref.key);
    }
};

class RandomHashFunction : public HashFunction
{
    public:
    /**
     * Constructor.
     */
    RandomHashFunction(sizetype _hashsize);
    /**
     * Both hashfunctions.
     */
    sizetype h1(keytype _key);
    sizetype h2(keytype _key);
    /**
     * Returns a specifing string for this hashfunction.
     * @returns a C-Type string containing the specifier.
     */
    char* getSpecifier()
    {return const_cast<char*>("Fully Random");};
    /**
     * Provides Information about the maximal usable universe.
     */
    universetype getmaxUniverse()
    {return (universetype)(0xFFFFFFFFFFFFFFFF);};
    /**
     * Empty destructor. No memory is allocated that hast to be freed.
     */
    ~RandomHashFunction(){};
    private:
    /** 
     * a set of all key with their chosen hashvalues.
     */
    std::set<KeyFunction> p_set;
    /**
     * The range of both hashfunctions is defined by [0..p_m].
     */
    sizetype p_m;
};

#endif
