#ifndef HASH_FUNCTION_H_
#define HASH_FUNCTION_H_

#include "Definitions.h"
#include "time.h"

/**
 * This function is a interface and factory class. All Implemented hashfunctions are derived from this class.
 */
class HashFunction
{
public:
	/**
	* creates a instance of the Hashfunction.
	* @param _functiontype The id of the desired functiontype.
	* @param _hashsize the size of the Hashtable this function operates in.
	* @param _stashsize the maximal size of the stash
 	* @param _universesize the size of the universe of keys the function operates. 
	*/
	static HashFunction * create(uint32_t _functiontype, sizetype _hashsize, sizetype _stashsize=3, keytype _universesize = 0);
	
	/**
	* Interface for both hash functions
	*/
	virtual sizetype h1 (keytype _key)=0;
	virtual sizetype h2 (keytype _key)=0;
	
	
	virtual universetype getmaxUniverse()=0;

	/**
	 * Returns the name of the hashfunction
	 */
	virtual char* getSpecifier()=0;
	/**
	 * virtual destructor
	 */
	virtual ~HashFunction(){};	
};

#endif
