#ifndef SIMPLE_HASHING_H_
#define SIMPLE_HASHING_H_

#include "HashFunction.h"


/**
 * This class provides a simple hashfunction. Calculating a*x+b mod p mod m.
 * Where m defines the desired range of the function.
 * It uses the CW-trick shown in:
 * "Tabulation-Based 5-Independent Hashing with Applications to linear Probing and Second Moment Estimation",
 * M.Thorup, Y.Zhang, SIAM 2012
 */
class SimpleHashFunction:public HashFunction
{
public:
	 /**
	  * Both hashfunctions.
	  */
	 sizetype h1 (keytype _key);
	 sizetype h2 (keytype _key);

	 /**
	  * getter Functions for checking on the used parameters.
  	  * @returns the specified parameter
	  */
	 inline universetype get_a1(){return a1;};
	 inline universetype get_a2(){return a2;};
	 inline universetype get_b1(){return b1;};
	 inline universetype get_b2(){return b2;};
	 inline universetype get_m(){return m;};
	 inline universetype get_p(){return p;};
	/**
	*	Constructor usually called by the create function in "Hashfunction.cpp".
	*/
	SimpleHashFunction(sizetype _hashsize);

	/**
	 * Provides information about the maximal usable universesize.
	 * @returns The maximal usable universe with this hashfunction.
	 */
	universetype getmaxUniverse(){return p-1;};

	/**
	 * Returns a specifing sting for this hashfunction.
	 * @returns A c-type string containing the specifing string.
	 */
	char* getSpecifier(){return const_cast<char*>("CW-Trick 2 Independent");};
private:
	
	/**
	 * calculates a*x+b mod p. Possibly +2p or +p.
	 */
	inline universetype CWTrick(keytype x, universetype a, universetype b);
	
	/**
	* parameters used by both hashfunctions.
	* p the used prime.
	* m defines the range of the hashfunction.
	*/
	universetype	p;
	sizetype	m;
	/**
	 * Parameters exclusively used by hashfunction 1.
	 */
	universetype	a1,b1;
	/**
	 * Parameters exculsively used by hashfunction 2.
	 */
	universetype 	a2,b2;
};

#endif
