#ifndef TB_5U_32_T_8_H_
#define TB_5U_32_T_8_H_

#include "HashFunction.h"

class TB5U32t8 : public HashFunction
{
public:
	/**
	 * First hashfunction.
	 * @param _key The key this function is calculated for.
	 * @returns The hashvalue for the specified key.
	 */
	 sizetype h1 (keytype _key);
	/**
	 * Second hashfunction.
	 * @param _key the Key this function is calculated for.
	 * @returns The hashvalue for the specified key.
	 */
	 sizetype h2 (keytype _key);
	
	/**
	 * Constructor.
	 * @param _hashsize Defines the size of the hashtable this function is used for.
	 */
	 TB5U32t8(sizetype _hashsize);
	/**
	 * Destructor.
	 */
	 ~TB5U32t8();
	/**
	 * Returns a specifing string for this hashfunction.
	 * @returns a c-type string tha contains the specifier.
	 */
	 char* getSpecifier(){return const_cast<char*>("Tabulationbased 5 Universal 32to8");};

	 /**
	  * Provides information about the maximal usable universe with this hashfunction.
	  * @returns the maximal usable universe with this hashfunciton.
	  */
	universetype getmaxUniverse(){return 0xFFFFFFFF;};
private:
    
	/**
	 * Compresses a 32 bit number to a 30 bit number.
	 * For this purpose masks the number with several masks and combines the results.
  	 * @param i the number to be compressed.
	 * @returns the compressed number.
	 */
	inline sizetype compress(sizetype i);
	/**
	 * Defines the range of both hashfunctions.
	 */
	sizetype p_m;
	/**
	 * the tables used for calculating hashfunction 1
	 */
	sizetype **p_t01,
		 **p_t11,
		 **p_t21,
		 **p_t31;
	sizetype *p_t41,
		 *p_t51,
		 *p_t61;

	/**
	 * The tables used for calculating hashfunction 2
	 */
	sizetype **p_t02,
		 **p_t12,
		 **p_t22,
		 **p_t32;
	sizetype *p_t42,
		 *p_t52,
		 *p_t62;
};

#endif
