#ifndef TB5_UNIVERSAL_H_
#define TB5_UNIVERSAL_H_

#include "HashFunction.h"
/**
 * This class provides a 5-independent hashfunction.
 * This hashfunction was proposed in
 * "Tabulation-Based 5-Independent Hashing with Applications to Linear Probing
 *  and Second Moment Estimation",
 * M.Thorup,Y.Zhang, SIAM2012
 */
class TB5Universal:public HashFunction
{
public:	
	/**
	* Constructor creates a 5-Independent tabulationbased hashfunction.
	* Usually called by create function of "HashFunction.h."
	* @param _hashsize the maximal value this function can calculate.
	*/
	TB5Universal(sizetype _hashsize);
	/**
  	 * Destructor
	 */
	~TB5Universal();
	
	/**
	 * Both hash-functions.
	 */
	sizetype h1 (keytype _key);
	sizetype h2 (keytype _key);
	
	/**
	 * Provides information about the maximal universe size for this particular hashfunction.
	 * @returns The maximal key value this function can operate.
	 */
	universetype getmaxUniverse(){return 0xFFFFFFFF;};

	/**
	 * Returns a specifing string for this hashfunction.
	 * @returns A c-type String containing the specifing string.
	 */
	char* getSpecifier(){return const_cast<char*>("Tabulationbased 5 Universal 32to16");};
private:
	
	
	/**
	 * Tables for the first hash function.
	 */
	sizetype* p_t01;
	sizetype* p_t11;
	sizetype* p_t21;
	/**
	 * Tables for the second hash function.
	 */
	sizetype* p_t02;
	sizetype* p_t12;
	sizetype* p_t22;
	
	/**
	 * defines the range of both hashfunctions. 
	 */
	sizetype p_m;
  	/**
	 * Compresses a 32bit integer to 16bit
	 */
	inline keytype compressShort32(keytype i){return 2-(i>>16)+(i&0xFFFF);};
};

#endif
