#ifndef UI_5_UNIVERSAL_H_
#define UI_5_UNIVERSAL_H_

#include <HashFunction.h>

/**
 * A simple implementation of the 5 universal function ax^4+bx^3+cx^2+dx+e without any improvements
 * This function uses 2^32-5 as prime thus 2^32-6 is the maximal useable key
 */

class UI5Universal: public HashFunction
{
public:
	/**
	 * both hash functions
	 */
      	sizetype h1(keytype _key);
      	sizetype h2(keytype _key);
	/**
	 * Constructor.
	 * @param _hashsize Defines the maximal outputvalue of both functions.
	 */
      	UI5Universal(sizetype _hashsize);
	
	/**
	 * Provides information about the maximal usable universesize.
	 * @returns The maximal usable universe size.  
	 */
      	universetype getmaxUniverse(){return p_prime-1;};
	
	/**
	 * Function for checking on the used parameters for both functions.
	 * @param _cnt Will contain the length of the parameter list.
	 * @returns the parameter list.
	 */
	universetype* getParam(uint32_t& _cnt){
      	universetype * res = new universetype[12];
	_cnt=12;
	res[0]=p_a1;
	res[1]=p_b1;
	res[2]=p_c1;
	res[3]=p_d1;
	res[4]=p_e1;
	res[5]=p_a2;
	res[6]=p_b2;
	res[7]=p_c2;
	res[8]=p_d2;
	res[9]=p_e2;
	res[10]=p_prime;
	res[11]=p_m;
	return res;
      }

    	/**
	 * Returns a specifier string for this hash function.
	 * @returns a c-type string containing the specifier string.	
 	 */
	char* getSpecifier(){return const_cast<char*>("Unimproved 5 Universal");};
private:
	/**
	 * Parameters for both hashfunctions
	 */
      	universetype p_a1,p_b1, p_c1, p_d1, p_e1;
      	universetype p_a2,p_b2, p_c2, p_d2, p_e2;

	/**
	 * Common parameters for both functions.
	 * Used prime and the maximal output value.
	 */
      	universetype p_prime, p_m;
      
	/**
	 * calculates a*x+b.
	 * @param x the x value.
  	 * @param a the a value.
	 * @param b the b value.
	 * @returns a*x+b.
	 */
      	inline universetype basic(keytype x, universetype a, universetype b);
      
      
};

#endif
