#include "CW5Universal.h"
#include "rand_singleton.h"

sizetype CW5Universal::h1 (keytype _key)
{
	// calculates a*x^4 + b*x^3+c*x^2+d*x+e mod p
	universetype res = 
		    CWTrick(_key,
		      CWTrick(_key,
			CWTrick(_key,
			  CWTrick(_key,p_a1,p_b1)
			  ,p_c1)
			,p_d1)
		      ,p_e1);
	// the mod p for this mersenne prime
	res = (res&p_prime)+(res>>61);
	if(res >= p_prime)
		res-=p_prime;

	return (sizetype) (res %p_m);
}

sizetype CW5Universal::h2 (keytype _key)
{
	// calculates a*x^4 + b*x^3+c*x^2+d*x+e mod p
	universetype res = 
		    CWTrick(_key,
		      CWTrick(_key,
				CWTrick(_key,
					CWTrick(_key,p_a2,p_b2)
					,p_c2)
				,p_d2)
		      ,p_e2);
	//the mod p for this mersenne prime
	res = (res&p_prime)+(res>>61);
	if(res >= p_prime)
		res -= p_prime;
	return (sizetype) (res %p_m);
}

inline universetype CW5Universal::CWTrick(keytype _x, universetype _a, universetype _b){
	  universetype a0,a1,c0,c1;
	  // multiply x with the lower 32 bit of a
	  a0 = (_a&0xFFFFFFFF)*_x;
	  //multiply x with the upper 32 bit of a
	  a1 = (_a>>32)*_x;
	  // calculate the lower 64 bit of the results
	  c0 = a0 + (a1 << 32);
	  //calculate the bits 33..72 of the result
	  c1 = (a0>>32)+a1;
	  // modulo p for this merseene prime and + b
	  return( (c0 & p_prime)+(c1>>29)+_b );
}

CW5Universal::CW5Universal(sizetype _hashsize)
{
	// define the prime.
  	p_prime=((universetype)(1)<<61 )-1;
	// save the hashsize
	p_m=_hashsize;
	
	RandomGenerator* gen= RandomGenerator::get();	
	gen->range(0, p_prime-1);

	// choose all parameters randomly
	p_a1=gen->rand();
	p_b1=gen->rand();
	p_c1=gen->rand();
	p_d1=gen->rand();
	p_e1=gen->rand();
	p_a2=gen->rand();
	p_b2=gen->rand();
	p_c2=gen->rand();
	p_d2=gen->rand();
	p_e2=gen->rand();
}
