#include "CWkUniversal.h"
#include "rand_singleton.h"

sizetype CWkUniversal::h1 (keytype _key)
{
	//initialize res with the parameter corresponding to x^(k-1)
	universetype res = p_param1[0];
	//calculate the hashfunction
	for(uint32_t i=1; i<p_k;i++)
		res = CWTrick(_key,res,p_param1[i]);
	// the modulo operation for this mersenne prime  
	res = (res&p_prime)+(res>>61);
	if(res >= p_prime)
		res-=p_prime;

	return (sizetype) (res %p_m);
}

sizetype CWkUniversal::h2 (keytype _key)
{
	// intialize res with the parameter corresponding to x^(k-1)
	universetype res = p_param2[0];
	//calculate the hashfunction
        for(uint32_t i=1; i<p_k; i++)
       		res = CWTrick(_key, res, p_param2[i]);
        // the modulo operation for this mersenne prime
	res = (res&p_prime)+(res>>61);
	if(res >= p_prime)
		res -= p_prime;

	return (sizetype) (res %p_m);
}

inline universetype CWkUniversal::CWTrick(keytype _x, universetype _a, universetype _b){
	  universetype a0,a1,c0,c1;
	//multiply x with the lower 32bit of a
	  a0 = (_a&0xFFFFFFFF)*_x;
	// multiply x with the upper 32bit of a
	  a1 = (_a>>32)*_x;
	//the first 64 bit of the result
	  c0 = a0 + (a1 << 32);
	//bits 33..72 of the result
	  c1 = (a0>>32)+a1;
	// the modulo operation for this mersenne prime and addition of b
	  return( (c0 & p_prime)+(c1>>29)+_b );
}

CWkUniversal::CWkUniversal(sizetype _hashsize, uint32_t _universality)
{
	// save the universality
	p_k = _universality;
	// a minimum universality of 2 is neccessary
	if(p_k<2)
		p_k=2;
	//used mersenne prime is 2^61-1
  	p_prime=((universetype)(1)<<61 )-1;
	//store the hashsize
	p_m=_hashsize;
	
	RandomGenerator* gen= RandomGenerator::get();	
	gen->range(0, p_prime-1);
	
	// intialize the lists of parameters
	p_param1 = new universetype[p_k];
	p_param2 = new universetype[p_k];
	for(uint32_t i =0; i<p_k; i++)
	{
		//fill these lists with random parameters
		p_param1[i]=gen->rand();
		p_param2[i]=gen->rand();
	}
}

CWkUniversal::~CWkUniversal()
{
	//free the space allocated for the parameters
	delete[] p_param1;
	delete[] p_param2;
}
