#include "DAC1.h"
#include "rand_singleton.h"

uint32_t DAC1::round_up(double x)
{
	// cast the inputvalue to an integer ignores any fracture
	uint32_t y = (uint32_t)(x);
	// if the cast value is smaller than the original
	// round up has to add one to the cast value
	y = (x>y)?(y+1):y;
	return y;
}

sizetype DAC1::h1 (keytype _key)
{
	//calculate the direct calculated part of the hashfunction
	universetype res = p_f->h1(_key);
	//calculate the corresponding position in the parameter list
	uint32_t pos = p_g->h1(_key);
	// read the stored part from the parameter list
	res += p_param1[pos];
	
	return (sizetype) (res %p_m);
}

sizetype DAC1::h2 (keytype _key)
{
        //calculate the direct calculated part of the hashfunction
	universetype res = p_f->h2(_key);
	// calculate the corresponding position in the parameter list
	uint32_t pos = p_g->h1(_key);
	// read the stored part from the parameter list
	res += p_param2[pos];
	
	return (sizetype) (res %p_m);

}


DAC1::DAC1(sizetype _hashsize, keytype _universesize, sizetype _stashsize, double _delta)
{

	//store hashsize
 	p_m=_hashsize;
	// calculate the length of both parameter lists.
	double l = pow(_hashsize, _delta);
	p_j = round_up(l);
	// calculate the k needed to provide the parameter c=1
	double c = (_stashsize+2)/(_delta);
	
	uint32_t k = round_up(c);
	//the used universality for the hasfunctions is 2*k
	k*=2;
	
	// Hack to set degree of independence in a uniform way
	// see HashFunction.cpp
	k+=1000;


	//initialize all used hashfunctions
	p_f = HashFunction::create(k,p_m);
	
	p_g= HashFunction::create(k,p_j);
	
	
	RandomGenerator* gen = RandomGenerator::get();	
	gen->range(0, p_m);
	
	//initialize the 2 parameter lists
	p_param1 = new universetype[p_j];
	p_param2 = new universetype[p_j];
        			
	for(uint32_t j=0; j<p_j;j++)
	{
		//fill the 2 parameter lists with random parameters.
		p_param1[j]=gen->rand();
		p_param2[j]=gen->rand();
	}
	
}

DAC1::~DAC1()
{
	//frees all allocated memory
	if(p_g!=NULL)
	{
		delete p_g;
		p_g=NULL;
	}
	if(p_param1!=NULL)
	{	
		delete[] p_param1;
		p_param1=NULL;
	}
	if(p_param2!=NULL)
	{
		delete[] p_param2;
		p_param2=NULL;
	}
	if(p_f !=NULL)
	{
		delete p_f;
		p_f = NULL;
	}
}
