#include "DAK1.h"
#include "rand_singleton.h"

uint32_t DAK1::round_up(double x)
{
	// cast the double value to int ignores any fractions
	uint32_t y = (uint32_t)(x);
	// if x had any fractions we have to increase the resulting integer by one
	y = (x>y)?(y+1):y;
	return y;
}

sizetype DAK1::h1 (keytype _key)
{
	// calculate the result of the first hashfunction f
	universetype res = p_f->h1(_key);
	// read the rest of the hashfunction from the corresponding list
	for(uint32_t i=0; i <p_C;i++)
	{
		// every hashfunction provides 2 functions determine which one to use
		uint32_t pos = i%2 ? p_g[i/2]->h1(_key) : p_g[i/2]->h2(_key);
		// read the value from the corresponding list
		res += p_param1[i][pos];
	}
	return (sizetype) (res %p_m);
}

sizetype DAK1::h2 (keytype _key)
{
        // calculate the result of the first hashfunction f      
	universetype res = p_f->h2(_key);
	//read the rest of the value from the parameter lists
	for(uint32_t i=0; i <p_C;i++)
	{
		// every hashfunction provides 2 functions determine which one to use
		uint32_t pos = i%2 ? p_g[i/2]->h1(_key) : p_g[i/2]->h2(_key);
		// read the value from the corresponding list
		res += p_param2[i][pos];
	}
	return (sizetype) (res %p_m);

}


DAK1::DAK1(sizetype _hashsize, keytype _universesize, sizetype _stashsize, double _delta)
{
	// store the hashsize
 	p_m=_hashsize;
	// calculate the size of the parameter list used
	double l = pow(_hashsize, _delta);
	// calculate the number of parameter lists used
	double c = (_stashsize+2)/(_delta * 1);
	p_C = round_up(c);
	
	uint32_t k = round_up(l);
	uint32_t m=10000,n=0;
	
	//find next power of two above l 
	while(k!=0)
	{
		if(k%2!=0)
			m = (m==10000)?n:n+1;
		k/=2;
		n++;
	}
	//the actual size of the used parameter lists
	p_j = 1<<m;

	// the hashfunction for the direct calculated part of the hashfunction
	p_f = HashFunction::create(0,p_m);
	// the hashfunctions for the lookup in the lists
	p_g = new HashFunction*[(p_C/2)+(p_C%2)];

	for(uint32_t i=0; i<(p_C/2+(p_C%2));i++)
	{
		// create the hashfunctions for the lookup
		p_g[i]= HashFunction::create(4,p_j);
	}
	
	RandomGenerator* gen = RandomGenerator::get();	
	gen->range(0, p_m);
	
	//create p_C corresponding many lists
	p_param1 = new universetype*[p_C];
	p_param2 = new universetype*[p_C];
        			
	for(uint32_t i =0; i<p_C; i++)
	{
		// initialize each list
		p_param1[i]= new universetype[p_j];
		p_param2[i]= new universetype[p_j];

		for(uint32_t j=0; j<p_j;j++)
		{
			// store parameters for each list
			p_param1[i][j]=gen->rand();
			p_param2[i][j]=gen->rand();
		}
	}
}

DAK1::~DAK1()
{
	for(uint32_t i=0; i<p_C; i++)
	{	
		// free the space allocated for each list
		delete[] p_param1[i];
		delete[] p_param2[i];
	}
	// free the allocated hashfunctions
	for(uint32_t i=0;i<(p_C/2 + p_C%2);i++)
		delete p_g[i];
	delete p_f;
	
	// free the allocated memory for the pointers
	delete[] p_g;
	delete[] p_param1;
	delete[] p_param2;
}
