#include "DAK2.h"
#include "rand_singleton.h"

uint32_t DAK2::round_up(double x)
{
	//cast the inputvalue to an integer. 
	// any fracture part is ignored
	uint32_t y = (uint32_t)(x);
	// if the input value is greater than the cast value
 	// the cast value has to be incremented
	y = (x>y)?(y+1):y;
	return y;
}

sizetype DAK2::h1 (keytype _key)
{
	// calculated the direct calculated part of the hashfunction.
	universetype res = p_f->h1(_key);
	//read the stored part from the parameter lists.
	for(uint32_t i=0; i <p_C;i++)
	{
		// calculate the corresponding position in the ith parameter list.
		uint32_t pos = i%2 ? p_g[i/2]->h1(_key) : p_g[i/2]->h2(_key);
		//read the corresponding value from the ith parameter list.
		res += p_param1[i][pos];
	}
	return (sizetype) (res %p_m);
}

sizetype DAK2::h2 (keytype _key)
{
        //calculate the direct calculated part of the hashfunction.
	universetype res = p_f->h2(_key);
	//read the stroed part from the parameter list
	for(uint32_t i=0; i <p_C;i++)
	{
		//calculate the corresponding position in the ith parameter list.
		uint32_t pos = i%2 ? p_g[i/2]->h1(_key) : p_g[i/2]->h2(_key);
		//read the corresponding value from the ith parameter list.
		res += p_param2[i][pos];
	}
	return (sizetype) (res %p_m);

}


DAK2::DAK2(sizetype _hashsize, keytype _universesize, sizetype _stashsize, double _delta)
{

 	p_m=_hashsize;
	//calculate the length of each parameter list.
	double l = pow(_hashsize, _delta);
	p_j = round_up(l);
	//calculate the number of parameter lists
	double c = (_stashsize+2)/(_delta * 2);
	p_C = round_up(c);

	uint32_t m=10000,n=0;
	

	//create the hashfunctions for the direct calculated part.
	p_f = HashFunction::create(1,p_m);
	
	//create the hasfunctions for access in the parameter lists.
	p_g = new HashFunction*[(p_C/2)+(p_C%2)];

	for(uint32_t i=0; i<(p_C/2+(p_C%2));i++)
	{
		p_g[i]= HashFunction::create(1,p_j);
	}
	
	RandomGenerator* gen = RandomGenerator::get();	
	gen->range(0, p_m);
	//initialize the list of parameterlists.
	p_param1 = new universetype*[p_C];
	p_param2 = new universetype*[p_C];
        			
	for(uint32_t i =0; i<p_C; i++)
	{
		//initialize the parameterlists
		p_param1[i]= new universetype[p_j];
		p_param2[i]= new universetype[p_j];

		for(uint32_t j=0; j<p_j;j++)
		{
			//fill the parameter lists with random values.
			p_param1[i][j]=gen->rand();
			p_param2[i][j]=gen->rand();
		}
	}
}

DAK2::~DAK2()
{
	//frees all allocated memory
	for(uint32_t i=0; i<p_C; i++)
	{	
		delete[] p_param1[i];
		delete[] p_param2[i];
	}
	for(uint32_t i=0;i<(p_C/2 + p_C%2);i++)
		delete p_g[i];
	delete[] p_g;
	delete[] p_param1;
	delete[] p_param2;
	delete p_f;
}
