#include "TB5U32t8.h"
#include "rand_singleton.h"
sizetype TB5U32t8::h1 (keytype _key)
{
	sizetype* a0;
	sizetype* a1;
	sizetype* a2;
	sizetype* a3;
	sizetype c;
	a0=p_t01[(_key&0xFF)];
	a1=p_t11[((_key>>8)&0xFF)];
	a2=p_t21[((_key>>16)&0xFF)];
	a3=p_t31[((_key>>24)&0xFF)];
	c=a0[1]+a1[1]+a2[1]+a3[1];
	c=compress(c);
	return ((a0[0]^a1[0]^a2[0]^a3[0]
		^p_t41[(c&0x3FF)]
		^p_t51[((c>>10)&0x3FF)]
		^p_t61[((c>>20)&0x3FF)])%p_m);
}
sizetype TB5U32t8::h2 (keytype _key)
{
	sizetype* a0;
	sizetype* a1;
	sizetype* a2;
	sizetype* a3;
	sizetype c;
	a0=p_t02[(_key&0xFF)];
	a1=p_t12[((_key>>8)&0xFF)];
	a2=p_t22[((_key>>16)&0xFF)];
	a3=p_t32[((_key>>24)&0xFF)];
	c=a0[1]+a1[1]+a2[1]+a3[1];
	c=compress(c);
	return ((a0[0]^a1[0]^a2[0]^a3[0]
		^p_t42[c&0x3FF]
		^p_t52[(c>>10)&0x3FF]
		^p_t62[(c>>20)])%p_m);	
}
TB5U32t8::~TB5U32t8()
{
	// free all allocated memory
	for(unsigned int i=0; i<0x100;i++)
	{
		//free the 2 element lists.
		delete[] p_t01[i];
		delete[] p_t11[i];
		delete[] p_t21[i];
		delete[] p_t31[i];
		delete[] p_t02[i];
		delete[] p_t12[i];
		delete[] p_t22[i];
		delete[] p_t32[i];
	}
	delete[] p_t01;
	p_t01 = 0;
	delete[] p_t11;
	p_t11=0;
	delete[] p_t21;
	p_t21=0;
	delete[] p_t31;
	p_t31=0;
	delete[] p_t41;
	p_t41=0;
	delete[] p_t51;
	p_t51=0;
	delete[] p_t61;
	p_t61=0;
	  
	delete[] p_t02;
	p_t02 = 0;
	delete[] p_t12;
	p_t12=0;
	delete[] p_t22;
	p_t22=0;
	delete[] p_t32;
	p_t32 = 0;
	delete[] p_t42;
	p_t42 = 0;
	delete[] p_t52;
	p_t52 = 0;
	delete[] p_t62;
	p_t62 = 0;
}
TB5U32t8::TB5U32t8(sizetype _hashsize)
{
	p_m = _hashsize;
	//create all lists used for the loookup
	  p_t01=new sizetype*[0x100];
	  p_t11=new sizetype*[0x100];
	  p_t21=new sizetype*[0x100];
	  p_t31=new sizetype*[0x100];
	  p_t41=new sizetype[0x100];
	  p_t51=new sizetype[0x100];
	  p_t61=new sizetype[0x100];
	  
	  p_t02=new sizetype*[0x100];
	  p_t12=new sizetype*[0x100];
	  p_t22=new sizetype*[0x100];
	  p_t32=new sizetype*[0x100];
	  p_t42=new sizetype[0x100];
	  p_t52=new sizetype[0x100];
	  p_t62=new sizetype[0x100];
	  
	  RandomGenerator* gen=RandomGenerator::get();
	  gen->range(0, 0xFFFFFFFF);
	  for(int i=0; i<0x100;i++)
	  {
		
		p_t01[i]=new sizetype[2];
		p_t01[i][0]=gen->rand32();
		p_t01[i][1]=gen->rand32();
		p_t11[i]=new sizetype[2];
		p_t11[i][0]=gen->rand32();
		p_t11[i][1]=gen->rand32();
		p_t21[i]=new sizetype[2];
		p_t21[i][0]=gen->rand32();
		p_t21[i][1]=gen->rand32();
		p_t31[i]=new sizetype[2];
		p_t31[i][0]=gen->rand32();
		p_t31[i][1]=gen->rand32();
		p_t41[i]=gen->rand32();
		p_t51[i]=gen->rand32();
		p_t61[i]=gen->rand32();
		    
		p_t02[i]=new sizetype[2];
		p_t02[i][0]=gen->rand32();
		p_t02[i][1]=gen->rand32();
		p_t12[i]=new sizetype[2];
		p_t12[i][0]=gen->rand32();
		p_t12[i][1]=gen->rand32();
		p_t22[i]=new sizetype[2];
		p_t22[i][0]=gen->rand32();
		p_t22[i][1]=gen->rand32();
		p_t32[i]=new sizetype[2];
		p_t32[i][0]=gen->rand32();
		p_t32[i][1]=gen->rand32();
		p_t42[i]=gen->rand32();
		p_t52[i]=gen->rand32();
		p_t62[i]=gen->rand32();
	  }
}

inline sizetype TB5U32t8::compress(sizetype i)
{
  	const sizetype mask1 = (((sizetype) 0x3)<<20)+(((sizetype) 0x3)<<10)+ 0x3;
  	const sizetype mask2 = (((sizetype)0xFF)<<20)+(((sizetype)0xFF)<<10)+0xFF;
  	const sizetype mask3 = (((sizetype) 0x3)<<20)+(((sizetype) 0x3)<<10)+ 0x3;
  	return mask1 + (i&mask2) - ((i>>8)&mask3);
}
