#ifndef CUCKOO_HASHING_H_
#define CUCKOO_HASHING_H_

#include "Definitions.h"
#include "HashFunction.h"

class HashTable
{
public:
	/**
	*	Constructor. Creates the 2 hashtables. The size is based on the param _size. Creates the Stash size of the stash is defined by _stashsize 
	*	@param	_size		Defines the size of both hashtables by _size*epsilon
	*	@param _stashsize	Defines the size of the stash
	*	@param _universe	The universe the keys can come from is defined by {1.._universe}
	*	@param _looptype	Defines the used maximal number of loops 
	*	@param _functiontype Defines the type of the Hashfunctions that are going to be used
	*/
	HashTable(sizetype _size, sizetype _stashsize, keytype _universe, int _functiontype,int _looptype, float _epsilon);
	
	
	/**
	 * Destructor
	 */
	~HashTable();

public:
	
	/**
	*	Inserts Keys into the hashtables.
	*	@param _amount	defines the amount of keys that are going to be inserted
	*	@param _keys 	the keys that are going to be inserted
	* 	@returns average number of hashfunction calculations for one insert or 0 if rehash occured
	*/
	uint64_t fill(sizetype _amount, keytype* _keys);

	/**
	*	Inserts a key into the hashtable
	*	@param _key The key that is going to be inserted into the hashtable.
	*	@returns the number of hashfunction calculations. Returns 0 if a rehash occured.
	*/
	uint64_t insert(keytype _key);

	/**
	*	checks if a key is present in the hashing structure
	*	@param _key	the key you look for
	*	@returns 0 if the key is not present in the Hastables. 1 if the key is in Hashtable 1, 2 if it is present in hashtable 2 and 3 if it is stored in the stash.
	*/
	int lookup(keytype _key);

	/**
	 * Writes the content of the hashtable to the console.
	 */
	void print();
	
	/**
	 * Returns the hashfunction used by this hashtable.
	 * @returns the hashfuntion used by this hashtable.
	 */
	HashFunction* getHashfunction(){return p_function;};
	
	/**
	 * Returns the number of stashspaces in use.
	 * @returns the height of the stash.
	 */
	inline sizetype get_stashheight(){return p_stashheight;};
private:
	/**
	* Not implemented
	*/
	void rehash();
private:
	/**
	* first hashtable
	*/
	keytype* p_hashtable1;

	/**
	* second hashtable
	*/
	keytype* p_hashtable2;

	/**
	* the stash is storing keys to solve deadlock situations
	*/
	keytype* p_stash;
	
	/**
	 * defines the size of the universe the keys can come from
	 */
	keytype p_universe;

	/**
	* stores how many keys are currently in the stash
	*/
	sizetype p_stashheight;

	/**
	* defines how many keys can be stored in the stash
	*/
	sizetype p_stashsize;
	
	/**
	 * defines how many keys can be stored in each table
	 */
	sizetype p_tablesize;

	/**
	* defines the number of replacings before putting a key into the stash
	*/
	sizetype p_max_loop;

	HashFunction *p_function;	
};

#endif
