#ifndef CONVERSION_H_
#define CONVERSION_H_
#include "Definitions.h"

template<typename T>
/**Converts a c-string to a integer value.
 * The c-String has to be a decimal number. It expects a integerlike datatype as template and converts the string to the given datatype.
 * @param str A C-String repesenting a decimal number.
 */
T a2i(char* str)
{
	T zw=0;
	bool neg=false;
	uint32_t i=0;
	if(str[i]=='-')
	{
	      i++;
	      neg=true;
	}
	for(i; str[i]!=0;i++)
	{
		zw*=10;
		zw +=(str[i]-'0');
	}
	return (neg?zw*(-1):zw);
}

/**
 * Converts a given integer into a hex string of a defined size.
 * Allways the whole buffer is used leading zeros are added.
 * @param _buf the buffer that will contain the resulting string.
 * @param _num the number that has to be converted.
 * @param _length the length of the buffer.
 */
void i2h(char* _buf, uint64_t _num, uint32_t _length);

/**
 * Writes helpinformations to console.
 */
void help();

/**
 * Selects a set of keys from the defined universe. 
 * @param _amount defines the size of the set.
 * @param _universesize defines the maximum key
 * @returns a pointer to a unsorted list containing all keys of the set.
 */
keytype* chooseKey(sizetype _amount, keytype _universesize);

/** Executes the experiments for every implemented Funtiontype.
 * Repeats several steps _cnt times. These Steps are: 1. choose new keys 2. create a hashtable for each hashfunction. 3. Fill each Hashtable with the chosen keys 4. (TODO) Save results to file.
 * @param _hashtablesize Defines the size of the hashtables that are going to be used
 * @param _stashsize Defines the size of the used stash
 * @param _universesize Defines the maximum key
 * @param _epsilon defines the used epsilon by _epsilon/10000
 * @param _cnt Defines how often the experiments are executed
 * @param _ofile Defines the outputfile
 */
void experiment(sizetype _hashtablesize, sizetype _stashsize, keytype _universesize, uint32_t* _functiontype, uint32_t _numfunction, uint32_t _epsilon, int _looptype, uint32_t _cnt, char* _ofile);

/**
 * Executes a single Tablefill with a specified function
 * @param _hashtablesize Defines the size of the hashtables that are going to be used
 * @param _stashsize Defines the size of the used stash
 * @param _universesize Defines the maximum key
 * @param _functiontype A list containing all functiontypes that are goind to take part in the experiments
 * @param _numfunction the number of functiontypes the _functiontype list contains
 * @param _epsilon defines the used epsilon by _epsilon/10000
 * @param _functiontype Defines the function that is going to be tested
 */
void singletest(sizetype _hashtablesize, sizetype _stashsize, keytype _universesize, uint32_t _epsilon, int _looptype, uint32_t _functiontype);

/**
 * Executes tests for a set of functions. These tests consist of creating a function of the choosen type and calculating h1 and h2 for the values from 1 to 2^32-1.
 * The time needed for calculating is stored in the file
 * @param _functiontype a list of all functiontypes to be tested.
 * @param _numfunction the length of the list
 * @param _amount the number of calculations
 * @param _cnt the number of iterations
 * @param _ofile defines the outputfile
 */
void functiontest(uint32_t* _functiontype, uint32_t _numfunction, sizetype _amount, sizetype _cnt, char* _ofile);

#endif
