#include "Help.h"

#include <iostream>

void help()
{

	 std::cout<<     "Usage:\n"
		<<      "functionname [Options]\n"
		<<      "Options:\n"
		<<      "    -h           - displays this help screen\n"
		<<      "    -t <value>   - sets the size of both hashtables to <value>*(1+epsilon)\n"
		<<      "    -s <value>   - sets the size of the stash to <value>\n"
		<<      "    -u <value>   - reduces the key universe to {1..<value>}\n"
		<<      "    -e <value>   - sets epsilon to <value>/10000 \n"
		<<      "    -l <type>    - sets the loop used to <type>\n"
		<<      "                   Possible types:\n"
		<<      "                            0  - long loop iterates 2*hashtablesize+1\n"
		<<	    "                            1  - short loop iterates 3(s+2)* log(n)/log(1+epsilon) times" 
		<<      "    -f <type>    - sets the used hashfunction to <type>\n"
		<<      "                   Possible types:\n"
		<<      "                            0  - simple hashfunction a*x+b mod p mod m\n"
		<<      "                            1  - 5-independent Hashfunction using CW-trick\n"
		<<      "                            2  - Tabulation based 5-independent hashing 32bit key 16 bit characters\n"
		<<      "                            3  - basic 5-independent Hashfunction without improvements\n"
		<<      "                            4  - 2-universal a*x<<(l_in-l_out)\n"
		<<      "                            5  - Dietzfelbinger Aumueller k=1\n"
		<<      "                            6  - Dietzfelbinger Aumueller k=2\n"
		<<      "                            7  - Dietzfelbinger Aumueller c=1\n"
        <<      "                            8  - Tabulation based 5-independent hashing 32bit key to 8 bit characters\n"
        <<      "                            9  - full randomized hashfunction\n"
		<<      "                         1kkk  - k-independent Hashfunction using CW-Trick kkk-determines 3 digit decimal number\n"
  		<<	"    -x           - starts the experiments\n"
		<<	"                   Additional Options\n"
		<<	"			     all previous defined options work as well\n"
		<<	"                            -i <value>   - determines the number of experiments to execute\n"
		<<      "                            -f <type>	  - type now can contain more than 1 functiontype\n"
		<<	"    -o           - Defines the outputfile if one is created\n";
  

}

#include "rand_singleton.h"

#include <set>

keytype* chooseKey(sizetype _amount, keytype _universesize)
{
	//It's not possible to choose more then n keys from a [1..n] set
	if(_amount > _universesize)
	  return NULL;
	
	//allocates the memory needed to store the chosen elements
	keytype* keys = new keytype[_amount];

	//will contain the currently choosen element
	keytype element;
	
	//sets that contains all generated elements
	std::set<keytype> container;
	std::set<keytype>::iterator it;
		
	RandomGenerator* rg= RandomGenerator::get();
	rg->range(1, _universesize);
	
	//choosing the elements that are going to be inserted
	//this is needed to avoid inserting an element multiple times
	for(sizetype i=0; i<_amount; i++)
	{
		//choose a random element from [1.._universesize]
		element = rg->rand();

		//check if choosen element is allready present
		if(container.find(element)==container.end())
		{
		    container.insert(element);
		    keys[i]=element;
		}
		else
		{
		      //the i-th element has to be rechosen if it was allready present
		      i--;
		}
	}
	return keys;
}

#include "boost/chrono.hpp"
#include "CuckooHashing.h"
#include <fstream>

void experiment(sizetype _hashtablesize, sizetype _stashsize, keytype _universesize,uint32_t* _functiontype, uint32_t _numfunction, uint32_t _epsilon, int _looptype, uint32_t _cnt, char* _ofile)
{
	
	if(_universesize<_hashtablesize)
	{
	  std::cout<<"Can't execute if the universe is smaller than the number of keys to choose\n";
	  return;
	}
	
	//varaible to show the progress of the experiments
	int percent=0;
	int newpercent=0;
	
	
	std::cout<<"Begin Experiments, Epsilon:"<<_epsilon<<" Count"<<_cnt<<" Hashtablesize:"<<_hashtablesize<<"\n";
	std::cout<<"Progress:"<<std::flush;
	
	//the pointer to the array of the keys
	keytype* keys;

	//will contain the maximum number of swaps in one fill process
	uint64_t res=0;
	
	float epsilon = (float)(_epsilon)/10000.0;
	
	//elapsed time
	boost::chrono::duration<int64_t, boost::nano> ela;

	boost::chrono::system_clock::time_point start;
	
	//c-type strings for storing the results that are going to be written to the outputfile
	char fth[4],lt[1],time[16],stash[8],ts[8],ss[8],us[8], nr[8], cnt[8], fts[4], eps[4],cal[16];
	
	
	i2h(lt, _looptype, 1);
	i2h(ts, _hashtablesize,8);
	i2h(ss, _stashsize, 8);
	i2h(us, _universesize,8);
	i2h(cnt, _cnt, 8);
	i2h(fts, _numfunction, 4);
	i2h(eps, _epsilon, 4);
	std::fstream file(_ofile,std::ios::trunc|std::ios::out);

	if(!file.is_open()){
		std::cout<<"Unable to open file.\n";
		return;
	}
	
	//write header to outputfile
	file.write(":",1);
	file.write(cnt,8);
	file.write(fts,4);
	file.write(ts, 8);
	file.write(ss, 8);
	file.write(us, 8);
	file.write(eps, 4);
	file.write(lt, 1);
	file.write("\n", 1);
	file.close();

	HashTable* table=NULL;
	for(uint32_t i=0; i<_cnt; i++)
	{
		keys = chooseKey(_hashtablesize, _universesize);
		//repeat for every Functiontype
		for(int ft=0; ft<_numfunction;ft++)
		{
			//create Hashtable;
			table = new HashTable(_hashtablesize,_stashsize,_universesize, _functiontype[ft], _looptype, epsilon);
			// save time before start of the fill process
			start = boost::chrono::system_clock::now();
	                //fill hashtable with the choosen keys
			res=table->fill(_hashtablesize,keys);
			//calculate elapsed time
	                ela = boost::chrono::system_clock::now() - start;
			
			
			i2h(time,ela.count(),16);
			i2h(fth,_functiontype[ft],4);
			i2h(stash, table->get_stashheight(),8);
			i2h(nr,i,8);
			i2h(cal,res,16);

			delete table;
			table = NULL;
			

			file.open(_ofile,std::ios::app|std::ios::out);
			//write results of this run to the outputfile
			if(!file.is_open())
				std::cout<<"File error "<<i<<" "<<ft;
			else
			{
				file.write("#",1);
				file.write(nr,8);
				file.write(fth,4);
				file.write(time,16);
				file.write(stash,8);
				file.write(cal,16);
				file.write("\n",1);
			}
			
			file.close();
		}
		//calculate the percentage of the executed iterations
		newpercent=(int)((float)(i+1)/(float)(_cnt)*100);
		if(newpercent>percent)
		{
			percent=newpercent;
			if(percent%5==0)
				std::cout<<"|"<<std::flush;
			else	std::cout<<"."<<std::flush;
		}
		
		//free the space allocated for the keys
		delete[] keys;
	}
	std::cout<<"\n";
}

void i2h(char* _buf, uint64_t _num, uint32_t _length)
{
	for(int32_t i=_length-1; i>=0;i--)
	{
		char s=char (_num%16);
		_num/=16;
		_buf[i]= s>9?s+'A'-10 : s+'0';
	}
}

void singletest(sizetype _hashtablesize, sizetype _stashsize, keytype _universesize, uint32_t _epsilon, int _looptype, uint32_t _functiontype){
	if(_universesize<_hashtablesize)
	{
	  std::cout<<"Can't execute if the universe is smaller than the number of keys to choose\n";
	  return;
	}

	float epsilon = (float)(_epsilon) / 10000.0;

	std::cout<<"Stashsize="<<_stashsize<<"\nHashtablesize="<<_hashtablesize<<"\nUniversesize="<<_universesize<<"\nepsilon="<<epsilon
		  <<"\nlooptype="<<_looptype<<"\nfunctiontype=";
	
	keytype* keys = chooseKey(_hashtablesize,_universesize);
	
	//will contain the maximum number of swaps in the fill process
	uint64_t res=0;

	//create the Table
	HashTable table(_hashtablesize,_stashsize,_universesize, _functiontype, _looptype, epsilon);
	
	
	std::cout<< _functiontype << " " <<table.getHashfunction()->getSpecifier()<<"\nBegin to Fill HashTable\n"<<std::flush;
	
	boost::chrono::system_clock::time_point start = boost::chrono::system_clock::now();
	
	res=table.fill(_hashtablesize,keys);
	
	boost::chrono::duration<int64_t, boost::nano> ela = boost::chrono::system_clock::now() - start;
	
	std::cout<<"Elapsed Time:"<<ela.count()<<"\n";
	
	if(res==0)
		std::cout<<"rehash occured\n";
	else
		std::cout<<"succesfull tablefill. Used Stashsize:"<<table.get_stashheight()<<"\nSwaps needed(median):"<<res<<"\n";
	for(sizetype i=0; i<_hashtablesize;i++)
		if(!table.lookup(keys[i]))
			std::cout<<"Couldn't find key";	
	//table.print();
	//free allocated memory
	delete[] keys;

}

void functiontest(uint32_t* _functiontype, uint32_t _numfunction, sizetype _amount, sizetype _cnt, char* _ofile)
{
	HashFunction* p_function=NULL;
	
	keytype* keys = new keytype[_amount];
		
	RandomGenerator* rg= RandomGenerator::get();
	rg->range(1, 0xFFFFFFFF);
	//choosing the elements that are going to be used for the test
	for(sizetype i=0; i<_amount; i++)
		keys[i]= rg->rand();
	
	//time variables
	boost::chrono::duration<int64_t, boost::nano> ela;
	boost::chrono::system_clock::time_point start;
	
	//c-type strings storing the results that are going to be written to the outputfile
	char time[16], fth[4],it[8],kn[8],nr[8],fkt[4],idl[8];
	
	//opening the outputfile
	std::fstream file(_ofile,std::ios::trunc|std::ios::out);
	
	if(!file.is_open()){
		std::cout<<"Unable to open file.\n";
		return;
	}

	i2h(kn,_amount,8);
	i2h(it,_cnt,8);
	i2h(fkt,_numfunction,4);
	i2h(idl, 0,8);
	file.write("?",1);
	file.write(it,8);
	file.write(fkt,4);
	file.write(kn,8);
	file.write(idl,8);
	file.write("\n",1);
	file.write("\n",1);
	for(sizetype i=0; i<_cnt;i++ )
	for(uint32_t i=0; i < _numfunction; i++)
	{
	    //create the desired hashfunction
	    p_function = HashFunction::create(_functiontype[i],(sizetype)(0xFFFFFFFF));
	    
	    //store starting time
	    start = boost::chrono::system_clock::now();
	    
	    //calculate the function for every chosen key once
	    for(sizetype k=0; k<_amount;k++)
	    {
		p_function->h1(keys[k]);
	    }
	    
            //calculate elapsed time
	    ela = boost::chrono::system_clock::now() - start;
	    
	    i2h(time,ela.count(),16);
	    i2h(fth,_functiontype[i],4);
	    i2h(nr,i,8);
	    file.write("#",1);
	    file.write(nr,8);
	    file.write(fth,4);
	    file.write(time,16);
	    file.write("\n",1);
	    delete p_function;
	}
	delete[] keys;
}
